@@ -15,7 +15,6 @@
 # limitations under the License.
 
 import collections
-import logging
 
 from ryu.base import app_manager
 import ryu.exception as ryu_exc
@@ -79,14 +78,15 @@ class Networks(dict):
 
     def remove_network(self, network_id):
         try:
-            network = self[network_id]
+            ports = self[network_id]
         except KeyError:
             raise NetworkNotFound(network_id=network_id)
 
-        for (dpid, port_no) in network:
-            self.send_event(EventNetworkPort(network_id, dpid, port_no, False))
-        self.send_event(EventNetworkDel(network_id))
-        del self[network_id]
+        while ports:
+            (dpid, port_no) = ports.pop()
+            self._remove_event(network_id, dpid, port_no)
+        if self.pop(network_id, None) is not None:
+            self.send_event(EventNetworkDel(network_id))
 
     def list_ports(self, network_id):
         try:
@@ -107,10 +107,14 @@ class Networks(dict):
     #     self.add_raw(network_id, dpid, port_no)
     #     self.add_event(network_id, dpid, port_no)
 
+    def _remove_event(self, network_id, dpid, port_no):
+        self.send_event(EventNetworkPort(network_id, dpid, port_no, False))
+
     def remove_raw(self, network_id, dpid, port_no):
-        if (dpid, port_no) in self[network_id]:
-            self.send_event(EventNetworkPort(network_id, dpid, port_no, False))
-            self[network_id].remove((dpid, port_no))
+        ports = self[network_id]
+        if (dpid, port_no) in ports:
+            ports.remove((dpid, port_no))
+            self._remove_event(network_id, dpid, port_no)
 
     def remove(self, network_id, dpid, port_no):
         try:
@@ -168,13 +172,12 @@ class DPIDs(dict):
         try:
             # self.dpids[dpid][port_no] can be already deleted by
             # port_deleted()
-            port = self[dpid].get(port_no)
+            port = self[dpid].pop(port_no, None)
             if port and port.network_id and port.mac_address:
                 self.send_event(EventMacAddress(dpid, port_no,
                                                 port.network_id,
                                                 port.mac_address,
                                                 False))
-            self[dpid].pop(port_no, None)
         except KeyError:
             raise PortNotFound(dpid=dpid, port=port_no, network_id=None)
 
@@ -318,6 +321,12 @@ class Network(app_manager.RyuApp):
     def list_ports(self, network_id):
         return self.networks.list_ports(network_id)
 
+    def list_ports_noraise(self, network_id):
+        try:
+            return self.list_ports(network_id)
+        except NetworkNotFound:
+            return []
+
     def _update_port(self, network_id, dpid, port, port_may_exist):
         def _known_nw_id(nw_id):
             return nw_id is not None and nw_id != self.nw_id_unknown
@@ -365,7 +374,12 @@ class Network(app_manager.RyuApp):
         old_mac_address = self._get_old_mac(network_id, dpid, port_no)
 
         self.dpids.remove_port(dpid, port_no)
-        self.networks.remove(network_id, dpid, port_no)
+        try:
+            self.networks.remove(network_id, dpid, port_no)
+        except NetworkNotFound:
+            # port deletion can be called after network deletion
+            # due to Openstack auto deletion port.(dhcp/router port)
+            pass
         if old_mac_address is not None:
             self.mac_addresses.remove_port(network_id, dpid, port_no,
                                            old_mac_address)
