@@ -15,10 +15,10 @@
 
 import struct
 from . import packet_base
-from ryu.lib import mac
+from ryu.lib import addrconv
 
 # Slow Protocol Multicast destination
-SLOW_PROTOCOL_MULTICAST = '\x01\x80\xc2\x00\x00\x02'
+SLOW_PROTOCOL_MULTICAST = '01:80:c2:00:00:02'
 
 # Slow Protocol SubType
 SLOW_SUBTYPE_LACP = 0x01
@@ -88,53 +88,57 @@ class lacp(packet_base.PacketBase):
     | Version Number                                 | 1      |
     +------------+-----------------------------------+--------+
     | TLV        | TLV_type = Actor Information      | 1      |
-    | Actor      +-----------------------------------+--------+
+    | Actor      |                                   |        |
+    +------------+-----------------------------------+--------+
     |            | Actor_Information_Length = 20     | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_System_Priority             | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_System                      | 6      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_Key                         | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_Port_Priority               | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_Port                        | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Actor_State                       | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Reserved                          | 3      |
     +------------+-----------------------------------+--------+
     | TLV        | TLV_type = Partner Information    | 1      |
-    | Partner    +-----------------------------------+--------+
+    | Partner    |                                   |        |
+    +------------+-----------------------------------+--------+
     |            | Partner_Information_Length = 20   | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_System_Priority           | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_System                    | 6      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_Key                       | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_Port_Priority             | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_Port                      | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Partner_State                     | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Reserved                          | 3      |
     +------------+-----------------------------------+--------+
     | TLV        | TLV_type = Collector Information  | 1      |
-    | Collector  +-----------------------------------+--------+
+    | Collector  |                                   |        |
+    +------------+-----------------------------------+--------+
     |            | Collector_Information_Length = 16 | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Collector_Max_Delay               | 2      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Reserved                          | 12     |
     +------------+-----------------------------------+--------+
     | TLV        | TLV_type = Terminator             | 1      |
-    | Terminator +-----------------------------------+--------+
+    | Terminator |                                   |        |
+    +------------+-----------------------------------+--------+
     |            | Terminator_Length = 0             | 1      |
-    |            +-----------------------------------+--------+
+    +------------+-----------------------------------+--------+
     |            | Reserved                          | 50     |
     +------------+-----------------------------------+--------+
 
@@ -185,7 +189,9 @@ class lacp(packet_base.PacketBase):
     An instance has the following attributes at least.
     Most of them are same to the on-wire counterparts but in host byte
     order.
-    __init__ takes the correspondig args in this order.
+    __init__ takes the corresponding args in this order.
+
+    .. tabularcolumns:: |l|L|
 
     =============================== ====================================
     Attribute                       Description
@@ -209,60 +215,74 @@ class lacp(packet_base.PacketBase):
 
     actor_state_activity            .. _lacp_activity:
 
-                                    | about the activity control value
-                                      with regard to this link.
-                                    | LACP_STATE_ACTIVE(1)
-                                    | LACP_STATE_PASSIVE(0)
+                                    about the activity control value
+                                    with regard to this link.
+
+                                    LACP_STATE_ACTIVE(1)
+
+                                    LACP_STATE_PASSIVE(0)
 
     actor_state_timeout             .. _lacp_timeout:
 
-                                    | about the timeout control value
-                                      with regard to this link.
-                                    | LACP_STATE_SHORT_TIMEOUT(1)
-                                    | LACP_STATE_LONG_TIMEOUT(0)
+                                    about the timeout control value
+                                    with regard to this link.
+
+                                    LACP_STATE_SHORT_TIMEOUT(1)
+
+                                    LACP_STATE_LONG_TIMEOUT(0)
 
     actor_state_aggregation         .. _lacp_aggregation:
 
-                                    | about how the system regards this
-                                      link from the point of view of the
-                                      aggregation.
-                                    | LACP_STATE_AGGREGATEABLE(1)
-                                    | LACP_STATE_INDIVIDUAL(0)
+                                    about how the system regards this
+                                    link from the point of view of the
+                                    aggregation.
+
+                                    LACP_STATE_AGGREGATEABLE(1)
+
+                                    LACP_STATE_INDIVIDUAL(0)
 
     actor_state_synchronization     .. _lacp_synchronization:
 
-                                    | about how the system regards this
-                                      link from the point of view of the
-                                      synchronization.
-                                    | LACP_STATE_IN_SYNC(1)
-                                    | LACP_STATE_OUT_OF_SYNC(0)
+                                    about how the system regards this
+                                    link from the point of view of the
+                                    synchronization.
+
+                                    LACP_STATE_IN_SYNC(1)
+
+                                    LACP_STATE_OUT_OF_SYNC(0)
 
     actor_state_collecting          .. _lacp_collecting:
 
-                                    | about collecting of incoming
-                                      frames.
-                                    | LACP_STATE_COLLECTING_ENABLED(1)
-                                    | LACP_STATE_COLLECTING_DISABLED(0)
+                                    about collecting of incoming frames.
+
+                                    LACP_STATE_COLLECTING_ENABLED(1)
+
+                                    LACP_STATE_COLLECTING_DISABLED(0)
 
     actor_state_distributing        .. _lacp_distributing:
 
-                                    | about distributing of outgoing
-                                      frames.
-                                    | LACP_STATE_DISTRIBUTING_ENABLED(1)
-                                    | LACP_STATE_DISTRIBUTING_DISABLED(0)
+                                    about distributing of outgoing frames.
+
+                                    LACP_STATE_DISTRIBUTING_ENABLED(1)
+
+                                    LACP_STATE_DISTRIBUTING_DISABLED(0)
 
     actor_state_defaulted           .. _lacp_defaulted:
 
-                                    | about the Partner information
-                                      which the the Actor use.
-                                    | LACP_STATE_DEFAULTED_PARTNER(1)
-                                    | LACP_STATE_OPERATIONAL_PARTNER(0)
+                                    about the Partner information
+                                    which the the Actor use.
+
+                                    LACP_STATE_DEFAULTED_PARTNER(1)
+
+                                    LACP_STATE_OPERATIONAL_PARTNER(0)
 
     actor_state_expired             .. _lacp_expired:
 
-                                    | about the state of the Actor.
-                                    | LACP_STATE_EXPIRED(1)
-                                    | LACP_STATE_NOT_EXPIRED(0)
+                                    about the state of the Actor.
+
+                                    LACP_STATE_EXPIRED(1)
+
+                                    LACP_STATE_NOT_EXPIRED(0)
 
     partner_system_priority         The priority assigned to the
                                     Partner System.
@@ -366,9 +386,11 @@ class lacp(packet_base.PacketBase):
     _ALL_PACK_LEN = _HLEN_PACK_LEN + _ACTPRT_INFO_PACK_LEN * 2 + \
         _COL_INFO_PACK_LEN + _TRM_PACK_LEN
 
+    _MIN_LEN = _ALL_PACK_LEN
+
     def __init__(self, version=LACP_VERSION_NUMBER,
                  actor_system_priority=0,
-                 actor_system=mac.haddr_to_bin('00:00:00:00:00:00'),
+                 actor_system='00:00:00:00:00:00',
                  actor_key=0, actor_port_priority=0, actor_port=0,
                  actor_state_activity=0, actor_state_timeout=0,
                  actor_state_aggregation=0,
@@ -376,7 +398,7 @@ class lacp(packet_base.PacketBase):
                  actor_state_collecting=0, actor_state_distributing=0,
                  actor_state_defaulted=0, actor_state_expired=0,
                  partner_system_priority=0,
-                 partner_system=mac.haddr_to_bin('00:00:00:00:00:00'),
+                 partner_system='00:00:00:00:00:00',
                  partner_key=0, partner_port_priority=0, partner_port=0,
                  partner_state_activity=0, partner_state_timeout=0,
                  partner_state_aggregation=0,
@@ -537,14 +559,17 @@ class lacp(packet_base.PacketBase):
          ) = struct.unpack_from(cls._TRM_PACK_STR, buf, offset)
         assert cls.LACP_TLV_TYPE_TERMINATOR == terminator_tag
         assert 0 == terminator_length
-        return cls(version, actor_system_priority,
-                   actor_system, actor_key, actor_port_priority,
+        return cls(version,
+                   actor_system_priority,
+                   addrconv.mac.bin_to_text(actor_system),
+                   actor_key, actor_port_priority,
                    actor_port, actor_state_activity,
                    actor_state_timeout, actor_state_aggregation,
                    actor_state_synchronization, actor_state_collecting,
                    actor_state_distributing, actor_state_defaulted,
                    actor_state_expired, partner_system_priority,
-                   partner_system, partner_key, partner_port_priority,
+                   addrconv.mac.bin_to_text(partner_system),
+                   partner_key, partner_port_priority,
                    partner_port, partner_state_activity,
                    partner_state_timeout, partner_state_aggregation,
                    partner_state_synchronization,
@@ -558,13 +583,15 @@ class lacp(packet_base.PacketBase):
         actor = struct.pack(self._ACTPRT_INFO_PACK_STR,
                             self.actor_tag, self.actor_length,
                             self.actor_system_priority,
-                            self.actor_system, self.actor_key,
+                            addrconv.mac.text_to_bin(self.actor_system),
+                            self.actor_key,
                             self.actor_port_priority, self.actor_port,
                             self.actor_state)
         partner = struct.pack(self._ACTPRT_INFO_PACK_STR,
                               self.partner_tag, self.partner_length,
                               self.partner_system_priority,
-                              self.partner_system, self.partner_key,
+                              addrconv.mac.text_to_bin(self.partner_system),
+                              self.partner_key,
                               self.partner_port_priority,
                               self.partner_port, self.partner_state)
         collector = struct.pack(self._COL_INFO_PACK_STR,
