@@ -316,10 +316,10 @@ assert (calcsize(OFP_SWITCH_FEATURES_PACK_STR) + OFP_HEADER_SIZE ==
 OFPC_FLOW_STATS = 1 << 0    # Flow statistics.
 OFPC_TABLE_STATS = 1 << 1    # Table statistics.
 OFPC_PORT_STATS = 1 << 2    # Port statistics.
-OFPC_GROUP_STATS = 1 << 3    # 802.1d spanning tree.
+OFPC_GROUP_STATS = 1 << 3    # Group statistics.
 OFPC_IP_REASM = 1 << 5        # Can reassemble IP fragments.
 OFPC_QUEUE_STATS = 1 << 6    # Queue statistics.
-OFPC_PORT_BLOCKED = 1 << 8    # Match IP addresses in ARP pkts.
+OFPC_PORT_BLOCKED = 1 << 8    # Switch will block looping ports.
 
 # struct ofp_switch_config
 OFP_SWITCH_CONFIG_PACK_STR = '!HH'
@@ -768,6 +768,88 @@ OFPPR_ADD = 0    # The port was added.
 OFPPR_DELETE = 1    # The port was removed.
 OFPPR_MODIFY = 2    # Some attribute of the port has changed.
 
+# OFPMP_EXPERIMENTER
+# struct onf_experimenter_multipart_msg
+# (experimenter == ONF_EXPERIMENTER_ID)
+ONFMP_FLOW_MONITOR = 1870
+
+# EXT-187 seems to have a lot of flaws.
+# XXX the spec mentions ONFST_FLOW_MONITOR in some places.
+# we assume it's same as ONFMP_FLOW_MONITOR.
+# XXX the spec uses OFPP_NONE.  we assume it means OFPP_ANY.
+# XXX onf_flow_update_full.length is commented to be 24.
+# but it needs to tell the actual length of instructions.
+# we assume it's variable.
+# XXX the spec seems confused between instructions and actions
+# for onf_flow_update_full/ONFFMF_ACTIONS.  we assume they all
+# are instructions.
+# XXX the spec does not define payload structures for any of
+# ONFT_FLOW_MONITOR_CANCEL, ONFT_FLOW_MONITOR_PAUSED, or
+# ONFT_FLOW_MONITOR_RESUMED.  we assume they are same as NX.
+# according to NX spec (OVS nicira-ext.h and ofp-msg.h):
+#    NXT_FLOW_MONITOR_CANCEL: a single u32 'id'.
+#    NXT_FLOW_MONITOR_PAUSED/RESUMED: empty payload
+# (OF1.4 uses something different; OFPFMC_DELETE for CANCEL and
+# OFPFME_ for PAUSED/RESUMED.)
+# XXX onf_flow_monitor_request and onf_flow_update_full use
+# match_len + oxm_fields instead of ofp_match.  this pointless
+# diverge from OF1.4 looks like a botch when updating from OF1.0.
+# XXX the spec mentions "the current implementation of Open vSwitch"
+# but, as of writing this, it doesn't have this extension implemented
+# at all.  we assume that it is about OF1.0 NX.
+# XXX the spec mentions nx13_flow_monitor_request but i couldn't find
+# it in OVS nicira-ext.h.
+
+# onf_flow_monitor_request
+# ONFMP_FLOW_MONITOR request's body is zero or more instances of this.
+# id, flags, match_len, out_put, table_id, zeros[3]
+ONF_FLOW_MONITOR_REQUEST_PACK_STR = '!IHHIB3x'
+ONF_FLOW_MONITOR_REQUEST_SIZE = 16
+assert (calcsize(ONF_FLOW_MONITOR_REQUEST_PACK_STR) ==
+        ONF_FLOW_MONITOR_REQUEST_SIZE)
+
+# onf_flow_monitor_request.flags
+ONFFMF_INITIAL = 1 << 0
+ONFFMF_ADD = 1 << 1
+ONFFMF_DELETE = 1 << 2
+ONFFMF_MODIFY = 1 << 3
+ONFFMF_ACTIONS = 1 << 4
+ONFFMF_OWN = 1 << 5
+
+# onf_flow_update_header
+# ONFMP_FLOW_MONITOR request's body is an array of this
+# length, event
+ONF_FLOW_UPDATE_HEADER_PACK_STR = '!HH'
+ONF_FLOW_UPDATE_HEADER_SIZE = 4
+assert (calcsize(ONF_FLOW_UPDATE_HEADER_PACK_STR) ==
+        ONF_FLOW_UPDATE_HEADER_SIZE)
+
+# onf_flow_update_full, excluding onf_flow_update_header
+# reason, priority, idle_timeout, hard_timeout, match_len, table_id,
+# pad, cookie
+ONF_FLOW_UPDATE_FULL_PACK_STR = '!HHHHHBxQ'
+ONF_FLOW_UPDATE_FULL_SIZE = 24 - ONF_FLOW_UPDATE_HEADER_SIZE
+assert (calcsize(ONF_FLOW_UPDATE_FULL_PACK_STR) ==
+        ONF_FLOW_UPDATE_FULL_SIZE)
+
+# onf_flow_update_abbrev, excluding onf_flow_update_header
+# xid
+ONF_FLOW_UPDATE_ABBREV_PACK_STR = '!I'
+ONF_FLOW_UPDATE_ABBREV_SIZE = 8 - ONF_FLOW_UPDATE_HEADER_SIZE
+assert (calcsize(ONF_FLOW_UPDATE_ABBREV_PACK_STR) ==
+        ONF_FLOW_UPDATE_ABBREV_SIZE)
+
+# enum onf_flow_udpate_event
+ONFFME_ADDED = 0  # some variations in the spec; ONFMFE_ADD, ONFFME_ADD
+ONFFME_DELETED = 1
+ONFFME_MODIFIED = 2
+ONFFME_ABBREV = 3
+
+# enum onf_flow_monitor_msg_type
+ONFT_FLOW_MONITOR_CANCEL = 1870  # controller -> switch
+ONFT_FLOW_MONITOR_PAUSED = 1871  # switch -> controller
+ONFT_FLOW_MONITOR_RESUMED = 1872  # switch -> controller
+
 # struct ofp_error_msg
 OFP_ERROR_MSG_PACK_STR = '!HH'
 OFP_ERROR_MSG_SIZE = 12
@@ -798,11 +880,11 @@ OFPHFC_EPERM = 1        # Permissions error.
 # enum ofp_bad_request_code
 OFPBRC_BAD_VERSION = 0        # ofp_header.version not supported.
 OFPBRC_BAD_TYPE = 1        # ofp_header.type not supported.
-OFPBRC_BAD_MULTIPART = 2        # ofp_stats_msg.type not supported.
+OFPBRC_BAD_MULTIPART = 2        # ofp_multipart_request.type not supported.
 OFPBRC_BAD_EXPERIMENTER = 3    # Experimenter id not supported
                                # (in ofp_experimenter_header
-                               # or ofp_stats_request or
-                               # ofp_stats_reply).
+                               # or ofp_multipart_request or
+                               # ofp_multipart_reply).
 OFPBRC_BAD_EXP_TYPE = 4        # Experimenter type not supported.
 OFPBRC_EPERM = 5        # Permissions error.
 OFPBRC_BAD_LEN = 6        # Wrong request length for type.
@@ -973,6 +1055,32 @@ OFP_EXPERIMENTER_HEADER_SIZE = 16
 assert (calcsize(OFP_EXPERIMENTER_HEADER_PACK_STR) + OFP_HEADER_SIZE
         == OFP_EXPERIMENTER_HEADER_SIZE)
 
+# exp_type values for OFPET_EXPERIMENTER (experimenter=ONF_EXPERIMENTER_ID)
+ONFERR_ET_UNKNOWN = 2300
+ONFERR_ET_EPERM = 2301
+ONFERR_ET_BAD_ID = 2302
+ONFERR_ET_BUNDLE_EXIST = 2303
+ONFERR_ET_BUNDLE_CLOSED = 2304
+ONFERR_ET_OUT_OF_BUNDLES = 2305
+ONFERR_ET_BAD_TYPE = 2306
+ONFERR_ET_BAD_FLAGS = 2307
+ONFERR_ET_MSG_BAD_LEN = 2308
+ONFERR_ET_MSG_BAD_XID = 2309
+ONFERR_ET_MSG_UNSUP = 2310
+ONFERR_ET_MSG_CONFLICT = 2311
+ONFERR_ET_MSG_TOO_MANY = 2312
+ONFERR_ET_FAILED = 2313
+ONFERR_ET_TIMEOUT = 2314
+ONFERR_ET_BUNDLE_IN_PROGRESS = 2315
+ONFERR_ET_CANT_SYNC = 2320
+ONFERR_ET_BAD_PRIORITY = 2360
+ONFERR_ET_ASYNC_INVALUD = 2370
+ONFERR_ET_ASYNC_UNSUPPORTED = 2371
+ONFERR_ET_ASYNC_EPERM = 2372
+ONFERR_DUP_INSTRUCTION = 2600  # the lack of _ET_ is per spec
+ONFERR_ET_MPART_REQUEST_TIMEOUT = 2640
+ONFERR_ET_MPART_REPLY_TIMEOUT = 2641
+
 # struct ofp_hello
 OFP_HELLO_HEADER_SIZE = 8
 
@@ -1057,6 +1165,7 @@ oxm_types = [
     oxm_fields.OpenFlowBasic('pbb_isid', 37, oxm_fields.Int3),
     oxm_fields.OpenFlowBasic('tunnel_id', 38, oxm_fields.Int8),
     oxm_fields.OpenFlowBasic('ipv6_exthdr', 39, oxm_fields.Int2),
+    oxm_fields.ONFExperimenter('pbb_uca', 2560, oxm_fields.Int1),
 ]
 
 oxm_fields.generate(__name__)
